function [gU] = my_kergrad(U, K, KS, kernel_param)
    [n,m] = size(U);
    gU = zeros(n, m);
    if strcmp(kernel_param.type, 'linear')
        KU = K\U;
        gU = KU - KS*KU;
    else
        for k = 1:m
            % Compute KdC, KdD: the K's derivative wrt to C and D  
            repU = repmat(U(:,k), 1, size(U, 1));
            KdU = kernel_grad(K, repU, U, kernel_param,k);
            % the gradient of C
            Kd = K \ KdU;
            gU(:, k) = diag(Kd) - 0.5*dot(KS', Kd)' ;
        end
    end



function gradK = kernel_grad(KC, repC, C_all, kernel_param,k)
    switch kernel_param.type
        case 'rbf'
            gradK = -2 * kernel_param.gamma * (KC .* (repC' - repC));
%             KdD = -2 * kernel_param.gamma * (KD .* (repD' - repD));
        case 'poly'
            gradK = kernel_param.degree * (KC ./ (C_all*C_all' + 1) .* ...
                    (repC + diag(C_all(:,k))));
%             KdD = kernel_param.degree * (KD ./ (D_all*D_all' + 1) .* ...
%                     (repD + diag(D_all(:,k))));
        case 'linear'
            gradK = repC + diag(C_all(:,k));
%             KdD = repD + diag(D_all(:,k));
        case 'tapcov' % not finished!
        otherwise
            error('gradU: unknown kernel type');
    end